#!/usr/bin/python

#
# Tom Winegar Subaru Telescope - winegar@naoj.org
# unpack.py - client download program from STARS Query
#
# 2018-10-05 - add CURL option, wget -nv -a (not verbose), Download FITS Corrected, Download ASCII Corrections
#
# instructions for downloading:
# 1. create {new directory}
# 2. download S2Query.tar from email into {new directory}
# 3. from {new directory} execute 'tar -xvf S2Query.tar'
# 4. from {new directory} execute './zadmin/unpack.py'
# 5. files will be download to {new directory} without progess bar or notice
# 6. monitor the downbload in another window using 'ls -al' to list or 'ls -al | wc -l' to count 
#
# instructions for MD5 check and Missing file check after download:
# 1. execute ./zadmin/unpack.py again
# 2. Confirm MD5s on Existing Files? = Y
#
# instructions for downloading MD5 Fails and Missing files after downloading:
# 1. execute ./zadmin/unpack.py
# 2. Download Missing/MD5Fails files ONLY? = Y
#



import os,statvfs,shutil
import datetime
import subprocess
import os
import time

from string import maketrans
import hashlib


def getCWD() :

	cwd = os.getcwd() + '/'
	
	return ( cwd )
	
def BytestoText( diskfree ) :

	df=int(diskfree)/1000000
	dfs=str(df)+ 'MB'
	
	if df > 1000:
	
		df=int(diskfree)/1000000000
		dfs=str(df)+ ' GB'
				
		if df > 1000:
			df=int(diskfree)/1000000000000
			dfs=str(df)+ ' TB'
			
	return ( dfs )


def wGetRunning( ) :

	foundText=""

	cmd  = "ps -ef | grep 'wget --user='"
	
	fv = os.popen(cmd).read().strip()

#	print("fv: \n"+fv)	
	pslines = fv.split("\n")
	wget_ps = len( pslines ) - 2

	
	found = False
	
	if wget_ps > 0:
	
		found = True
		foundText = fv
			
	return ( found, foundText )

def curlRunning( ) :

	foundText=""

	cmd  = "ps -ef | grep 'curl -u'"
	
	fv = os.popen(cmd).read().strip()

#	print("fv: \n"+fv)	
	pslines = fv.split("\n")
	wget_ps = len( pslines ) - 2

	
	found = False
	
	if wget_ps > 0:
	
		found = True
		foundText = fv
			
	return ( found, foundText )

def getDomain( netfile ) :

	domain='none'
	
	if netfile == 'uhn.txt':
		domain = "http://stars2.naoj.hawaii.edu/data/"
	if netfile == 'uhns.txt':
		domain = "https://stars2.naoj.hawaii.edu/data/"
	if netfile == 'mtkn.txt' :
		domain = "http://www.mastars.nao.ac.jp/data/"
	if netfile == 'mtkns.txt' :
		domain = "https://www.mastars.nao.ac.jp/data/"
	if netfile == 'mtkin.txt' :
#		domain = "http://m2query.mastars.nao.ac.jp/data/"
		domain = "http://mquery.mastars.nao.ac.jp/data/"
	if netfile == 'oc3n.txt' :
		domain =  "http://squery.subaru.nao.ac.jp/data/"
	if netfile == 'oc3ns.txt' :
		domain =  "https://squery.subaru.nao.ac.jp/data/"

	if netfile == 'zip-uhn.txt':
		domain = "http://stars2.naoj.hawaii.edu/data/"
	if netfile == 'zip-uhns.txt':
		domain = "https://stars2.naoj.hawaii.edu/data/"

	if netfile == 'zip-oc3n.txt' :
		domain =  "http://squery.subaru.nao.ac.jp/data/"
	if netfile == 'zip-oc3ns.txt' :
		domain =  "https://squery.subaru.nao.ac.jp/data/"
	
	return ( domain )

def getDomain2() :

	print( "-- Choose Network and Archive --" )
#			print 'Network Routing Choices\n'
	print( "1. Hawaii Americas EU - UH network from Hawaii Archive - http" )
	print( "2. Hawaii Americas EU - UH network from Hawaii Archive - https" )
	print( "3. Japan Asia AU - University network from Mitaka Archive - http" )
	print( "4. Japan Asia AU - University network from Mitaka Archive - https" )
	print( "5. Mitaka NAOJ Campus - ADC network from Mitaka Archive - http" )
	print( "6. Subaru Internal - STN network from Hawaii Archive - http" )
	print( "7. Subaru Internal - STN network from Hawaii Archive - https" )
	
	response2=raw_input('Select Network/Archive? [1-7]: ')
	
	if response2 == '1':
		domain = "http://stars2.naoj.hawaii.edu/data/"
		
	if response2 == '2':
		domain = "https://stars2.naoj.hawaii.edu/data/"
		
	if response2 == '3' :
		domain = "http://www.mastars.nao.ac.jp/data/"
		
	if response2 == '4' :
		domain = "https://www.mastars.nao.ac.jp/data/"
		
	if response2 == '5' :
#		domain = "http://m2query.mastars.nao.ac.jp/data/"
		domain = "http://mquery.mastars.nao.ac.jp/data/"
		
	if response2 == '6' :
		domain =  "http://squery.subaru.nao.ac.jp/data/"
		
	if response2 == '7' :
		domain =  "https://squery.subaru.nao.ac.jp/data/"
		
	return ( domain )
	

def getQueryDat( cwd ) :

	datafile = 'zadmin/query.dat'

	fullpath = cwd + datafile

	if os.path.exists( fullpath ):

#		print( "\n" )	
#		print( "1st-Pass Query Data File exists at: "+fullpath+". reading ..." )
		g=open(fullpath,'r')

		for line in g: 
		
			querydata=line.split()
			qidno=querydata[0]
			qfiles=querydata[1]
			qsize=querydata[2]
			qcompsize=querydata[3]
			quser=querydata[4]
			qpw=querydata[5]
			qdir=querydata[6]
			qanon=querydata[7]
			qinstr=querydata[8]
			qdate=querydata[9]

#			print( "First Pass query.dat" )				
#			print( "Query IDNo: "+str(qidno) )			
#			print( "Query Files: "+str(qfiles) )			
#			print( "Query Size: "+BytestoText(qsize) )					
#			print( "Compressed Size: "+BytestoText(qcompsize) )					
#			print( "Download Anonymous: "+qanon )					
#			print( "Instrument: "+qinstr )					
#			print( "Date: "+qdate )	

		g.close()

	else:
		quser = ''			
		qpw = ''			
		qdir = ''			

	return ( quser, qpw, qdir, qfiles, qinstr )
	
def checkCRXDir( cwd ) :
	

	correxDir = cwd + 'zcorrex/'
	fitsDir = cwd
	
	#correxDir = '/stars2/html/zcorrex/'
	#fitsDir = '/stars2/html/'
	
	listfilesC = os.listdir ( correxDir )
	listfilesF = os.listdir ( fitsDir )
	
#	listfilesC2=listfilesC.sort()
	
#	listfilesF=listfilesF.sort()

	dirfiles=[]
		
	for item in listfilesC :
	
		if item is not None:
		
			dirfiles.append( item )
	
	dirfiles.sort()	
	
	for lineC in dirfiles :

#	correctionsArray = ()
		
#	for lineC in listfilesC2 :
		
		ext = lineC [-4:]
		
		frameID_C = lineC [0:12]
		
#		print ('Frame ID: ' + frameID_C )
		
		if ext == '.crx' :
		
			print ('Corrections File: ' + lineC)
			
#			correctionsArray.append( frameID_C )

#			for lineF in listfilesF :
#				print 'A-' +lineC 
#				ext = lineC[ -5: ]
#				frameID_F = lineF [0:12]
#				print 'A-'  + frameID_F 
				
#				if frameID_F == frameID_C:
#				
#					print ('Found match: ' + frameID_F + " " +  frameID_C )
#					
#					readOldFits( frameID_F, 'REQ-FWHM' )
					
					
				
#		else :
#			print ('Error: ' + ext)

#	correctionsArray.sort()	
	
#	print ("crx2" + str( correctionsArray ) )
	
	return( dirfiles )

def getkwCorrections ( allCorrectLines ) :
	
	kwCorrections = []
	
	AllCorrections = allCorrectLines.split("\n")

	for CorrectLine in AllCorrections :
	
		if len( CorrectLine ) > 0:
		
			kwCorrect = CorrectLine[0:8]
			
			if kwCorrect not in kwCorrections :
			
				kwCorrections.append( kwCorrect )

	return ( kwCorrections )
	
def checkValues ( char1, char2 ) :

	stringvalues = ['"',"'"]
	
	numvalues = ['0','1','2','3','4','5','6','7','8','9','-','.']

	checkvalue = True
	
	print ("1: " + char1 )
	print ("2: " + char2 )
	
	if char1 == char2 :
	
		checkvalue = False

	if ( char1 in numvalues and char2 in stringvalues ) or ( char2 in numvalues and char1 in stringvalues ) :

		checkvalue = False
		

	return ( checkvalue )
	
def writeCorrex( cwd, filename, AllCorrectLines ) :

#	print("in writeCorrex")

	write = 'yes'	
	#
#	oldfitsFile =  '/stars2/html/HSCA90939557.fits'
#	newfitsFile =  '/stars2/html/HSCA90939557-A.fits'


	
	fitsDir = cwd

	oldfitsFile = fitsDir + filename + '.fits'
	newfitsFile = fitsDir + filename + '-A.fits'
	
	kwCorrections = getkwCorrections ( AllCorrectLines )
	
#	keyword = CorrectLine[0:8]

#	kwCorrections = []

#	AllCorrections = AllCorrectLines.split("\n")

#	for CorrectLine in AllCorrections :
	
#		if len( CorrectLine ) > 0:
		
#			kwCorrect = CorrectLine[0:8]
#			if kwCorrect not in kwCorrections :
#				kwCorrections.append( kwCorrect )
	

	print("Corrections: " + str(kwCorrections))
	# test for file exists

	fits_bytes = 0
	grab_bytes = 80 
	
	m = hashlib.md5()
	
	if os.path.exists ( oldfitsFile ) :

	# read file
	# why readlines?

		print ("File exists: " + oldfitsFile)
		FILE2 = open ( oldfitsFile, 'rb' )
	#	listfiles = FILE2.readlines()

		if write == 'yes' :
	
			FILE3 = open ( newfitsFile, 'wb')
			
#		linemax = int( fitsline )
		linemax = 190
		lineseq = 0

		badkeys = ( 'COMMENT', 'HISTORY' )

#		while lineseq < linemax:

		while True :
		
			lineseq += 1

			listfiles2 = FILE2.read( grab_bytes )

#			if listfiles2 is None:
#			if len( listfiles2 ) == 0:

			if not listfiles2:
			
				break

			fits_bytes += grab_bytes

			lineLength = len( listfiles2 )

#			print( listfiles2 )

#			print("This " + str( lineseq )  + " line is : "+str( lineLength ) )

			kw = listfiles2[0:8]
			
			if not kw in kwCorrections:

#				print ('Copying ' + listfiles2 )
# Update md5 
				m.update ( listfiles2 )
		#
				if write == 'yes' :

					FILE3.write( listfiles2 )
					
			else: 
						
				AllCorrections = AllCorrectLines.split("\n")

				for FixLine in AllCorrections :
				
					if len ( FixLine ) > 0 :

						kwCorrect = FixLine[0:8]

						if kwCorrect == kw :

							CorrectLine = FixLine.strip()				
							lengthcorrectLine = len( CorrectLine )

							print("lengthCorrectLine: "+ str( lengthcorrectLine ))

							crxFragments = CorrectLine.split("=")

							keyword1 = crxFragments[0]
							values1 = crxFragments[1]

							values1 = values1.strip()

							lineFragments = listfiles2.split("=")

							keyword2 = lineFragments[0]
							values2 = lineFragments[1]
							oldvalue = values2.strip()
							strpos = oldvalue.find(' / ')


							values2 = values2.strip()

							comment = ''	

							if strpos > 0 :

								comment = oldvalue[ strpos:80 ] 

	#							comment = comment.strip()
								comment = comment.rstrip()

		#						print ( 'Comment from strpos: ' + comment ) 
		#						print ( 'length Correctline from strpos: ' + str( lengthcorrectLine ) ) 

								oldvalue = oldvalue [ 0:strpos ]

								oldvalue = oldvalue.strip()

		#						print ('Old value ' + oldvalue )


							lengthComment = len( comment )
		#					print("lengthComment: "+ str( lengthComment ))


							transtab = maketrans( '"', "'" )
							values1 = values1.translate( transtab )							


							if lengthComment == 0 :

		#						print("No Comment: "+ str( lengthComment ))


								if "'" in values1 or '"' in values1:

									CorrectLine3 = keyword1 + "= " + values1

								else:

									CorrectLine3 = keyword1 + "= " + values1.rjust( 20, " " )

								CorrectLine4 = CorrectLine3.ljust( 80, " " )

							else :

		#						print("Comment > 0: "+ str( lengthComment ))

								if "'" in values1 or '"' in values1:

									CorrectLine2 = keyword1 + "= " + values1.ljust( 20, " " )

									totalLineLength = len( CorrectLine2 ) + lengthComment

									if totalLineLength > 80 and lengthComment > 0 :

										trimLength = 80 - totalLineLength
										comment = comment[ 0 : trimLength ]
										lengthComment = len ( comment ) 

								else:

									CorrectLine2 = keyword1 + "= " + values1.rjust( 20, " " )

								CorrectLine3 = CorrectLine2 + comment

								CorrectLine4 = CorrectLine3.ljust( 80, " " )


							print ('Inserting ' + CorrectLine4 )

							if write == 'yes' :


								FILE3.write ( CorrectLine4 )
						
							m.update ( CorrectLine4 )

#
		FILE2.close()

		if write == 'yes' :

			FILE3.close()


	#	print("readline-1: :" + listfiles)
	#	print("readline-2: :" + listfiles2)


	#print ("filename: "+ oldfitsFile )

	print("fits_bytes: " + str( fits_bytes ) )


	print("MD5: " + m.hexdigest() )


def renameCorrex ( cwd, filename ) :

#	cwd = os.getcwd() + "/"
	fitsDir = cwd

	# make filename variable

	oldfitsFile = fitsDir + filename + '.fits'
	newfitsFile = fitsDir + filename + '-A.fits'
	
	if os.path.isfile ( oldfitsFile ) and os.path.isfile ( newfitsFile ) :

		# unlink ".fits" files
	
		os.unlink ( oldfitsFile )

		# rename "-A.fits" file to ".fits" file

		os.rename ( newfitsFile, oldfitsFile )
		
		print ( "Rename successful: " + oldfitsFile )

	else :
		print ( "Not a file: old: " + oldfitsFile + " / new: " + newfitsFile )

# print menu



def displayCorrex ( cwd ) :	

	correxDir = cwd + 'zcorrex/'
	
	listfilesC = os.listdir ( correxDir )
	
#	listfilesC2 = listfilesC.sort()

	dirfiles=[]
		
	for item in listfilesC :
	
		if item is not None:
		
			dirfiles.append( item )
	
	dirfiles.sort()	
	
	for lineC in dirfiles :

#	for lineC in listfilesC :
		
		ext = lineC [-4:]
		
		frameID_C = lineC [0:12]
		
		crxPath = correxDir + lineC
				
		if ext == '.crx' :
		
			print ('** Corrections File: ' + crxPath)
			
#			print ('** Corrections File: ' + lineC)
						
			allCorrections = getCorrex ( cwd, frameID_C )
			
			print ( allCorrections )
			
def getCorrex( cwd, filename ) :


	correxDir = cwd + 'zcorrex/'

#	filename = correxDir + 'HSCA90939557.crx'
	filename = correxDir + filename + '.crx'

#	filename = '/stars2/html/zcorrex/' + filename + '.crx'

	# 'w' - erase file before opening
	# 'a' append to file
	# 'r' read from file

	AllCorrectLines = ''
	
	if os.path.exists ( filename ) :

#		print( "File Exists: " + filename )

		FILE1 = open ( filename, 'rb' )
		listfiles = FILE1.readlines()
		FILE1.close()

#		print ( "getCorrecx.listfiles: " + str( listfiles )) 

		lineseq = 0

		for line in listfiles :

			lineseq += 1
			line = line.strip()

#			print( str( lineseq ) + " " + line)

			if lineseq == 2:

	# this is the linenumber in the oldfitsfile for Keyword

				fitsline = line[2:5]
#				print("fitsline: "+str( fitsline ) )

	# ignore comments

#			if line[0] == "#":

#				print ("This is a comment: " + line)
	# collect correctline
			if not line[0] == "#":

#			else:
#				# print keyword 

				CorrectLine = line
				
				AllCorrectLines += CorrectLine + "\n"
				
#				CorrectLine = CorrectLine.ljust( 80, " " )

				keyword = line[0:8] 
#				print (str (lineseq) + " keyword: [ " + keyword + " ] is not a comment: " + line)

				# print value

				value = line[9:15] 
#				print (str (lineseq) + " value: [ " + value + " ] is not a comment: " + line)



	else: 
		print( "File Not Exists: " + filename )

#	return ( CorrectLine, CorrecLines )
	return ( AllCorrectLines )

	

#def checkMD5s( cwd, qdir, netfile, quser, qpw, mnocheck ) :

def checkMD5s ( cwd ) :

	
#	response10=raw_input('Update MD5s values from STARS? (y/n): ')
	
	response10 = 'N' 
	
	if response10 == 'y' or response10 == 'Y' :

# change dir to /zadmin for the netfile download					
		newdir = cwd + 'zadmin/'

		os.chdir( newdir )
		
		domain = getDomain( netfile )
		
		if not domain == 'none' :
						
			netfileInput = domain + qdir + "/zadmin/zadmin/filelist.txt"											
			cmd='wget -N --user='+quser+' --password=' + qpw + mnocheck+ " " +netfileInput+" -a ./transfer.log"
			
			print( "MD5 update filelist.txt: " + cmd )
			
			os.system(cmd)
		
		else:
			print( "Domain not found: " + netfile )

# change dir back											
		os.chdir(cwd)
	

#	listpath30 = cwd+'/zadmin/zadmin/filelist.txt'
	listpath30 = cwd + 'zadmin/filelist.txt'

	missCount = 0
	failCount = 0
	goodCount = 0
	zipfzCount = 0
	fzCount = 0
	fitsCount = 0

	if os.path.exists( listpath30 ):

		# open filelist.txt
		FILE30 = open( listpath30 ,"r")
		listfiles30 = FILE30.readlines()
		FILE30.close()


		# for line in filelist.txt

		for line in listfiles30:

		# make values

		      
			fid2, fsize2, fmd5, fdate_obs, fzip = line.split(' ')
			fid=fid2.strip()
			fsize=int( fsize2.strip() )
			fmd5 = fmd5.strip().upper()

			if fzip == 'zipfz' :
				zipfzCount += 1
			if fzip == 'fz' :
				fzCount += 1
			if fzip == 'fits' :
				fitsCount += 1


			filepath2 = cwd + '/' + fid + '.fits'


		#								print 'chkfits filepath2' + filepath2 									
			mGet = False
# check Not os.path.exists
			if not os.path.exists( filepath2 ):

#			      mGet = True
				missCount = missCount + 1
				print( "Missing: "+filepath2 + " missingCount: " + str( missingCount ) )
#
# else check filebytes
			else:

				cmd = 'md5sum -b '+filepath2
				fv = os.popen(cmd).read().strip()
				realMD5 = fv.split(' ')
				actualMD5 = realMD5[0]
				actualMD5 = actualMD5.strip().upper()

				if not fmd5 == actualMD5:

					failCount = failCount + 1
#				      mGet = True

					print( "* MD5s NoMatch *: " + filepath2  + " failCount: " + str( failCount ) )
					print( fmd5 + " != " + actualMD5 )

				else:

					print( "MD5s Match: "+ fid + " == " + actualMD5 )

					goodCount=goodCount + 1

	print("\n-- MD5 Check Summary -- ") 
	print( str( missCount ) + " = Missing Files" )
	print( str( failCount ) + " = MD5-Check Fails" )
	print( str( goodCount ) + " = MD5-Check Confirmed" ) 

	return ( missCount, failCount, goodCount )



def findMD5 ( cwd, frameID, filename ) :

#	fitsDir = '/stars2/html/'

	zadmindir = cwd + 'zadmin/'
	
	filename1 = zadmindir + filename + '.txt'

	openFile1 = open( filename1, 'r' )
	readFile1 = openFile1.readlines()
	openFile1.close()
	
	foundFlag = False
	md5value = ''
		
	for matchFrame in readFile1 :
			
		if matchFrame [ 0:12 ] == frameID :
			
			foundFlag = True			
				
			matchArray = matchFrame.split (" ")
				
			md5value = matchArray [ 2 ]
			
#			print ("Inside findmd5" + matchFrame + md5value)

	return ( foundFlag , md5value )
	
def compareMD5 ( cwd ) :

#	correxDir = '/stars2/html/zcorrex/'

	listfilesF = os.listdir ( cwd )
	
	
	totalCount = 0
	successCount = 0
	failCount = 0


	dirfiles=[]
		
	for item in listfilesF :
	
		ext = item [-5:]
		
		if ext == '.fits' :
		
			dirfiles.append( item )
	
	dirfiles.sort()	
	
	for fitsFiles in dirfiles :
	
		totalCount += 1
	
		frameID = fitsFiles[0:12]
	
#		print frameID 
#		print fitsFiles
	

		fitsPath = cwd + fitsFiles
		
		openFile = open( fitsPath )
		
		readFile = openFile.read()
		
		
		md5Hash = hashlib.md5( readFile )
		md5Hashed = md5Hash.hexdigest()
		
		print fitsFiles + " calcmd5: " + md5Hashed
		
# def findMD5 ( frameid, filename ) / 'filelist', 'fitscorrex'

		filename = 'filelist'
	
		foundFlag1, md5value1 = findMD5 ( cwd, frameID, filename )

		filename = 'fitscorrex'

		foundFlag2, md5value2 = findMD5 ( cwd, frameID, filename )
		
		if ( foundFlag1 == True and md5Hashed == md5value1 ) or  ( foundFlag2 == True and md5Hashed == md5value2 ) :
		
			if foundFlag1 == True and md5Hashed == md5value1 :

				print ("Orig-Match: " + md5Hashed + " == " + md5value1 )
			
			if foundFlag2 == True and md5Hashed == md5value2 :
				
				print ("Crx-Match: " + md5Hashed + " == " + md5value2 )
			
			successCount += 1
		else :
				
			failCount += 1
			
			print ("Failure : " + frameID + " " + md5Hashed + " " + md5value1 )
								
		print ("Total: " + str( totalCount ) + " MD5 Success: " + str( successCount ) + " MD5 Fail: " + str( failCount ) )


def main():

	
	version='2018-10-05 unpack.py'
	

# os information

	mfail = 'F'
	
	transferCmd = 'wget'

	start_time = time.time()
	
	uname2=os.uname()
	uname=uname2[0]

	cwd = getCWD()

	print( "* Your Environment" )
	print( "STARS 2 Unpack ("+ version + ")\nOS: " + uname + " pwd: " + cwd )
#	print( "- Your Environment -" )
#	print( "*" )

#	print( " OS: "+uname )

	
#	print( "cwd: "+cwd )

# MacOS has no md5sum or md5sum -c or wget - so Fail

	if uname == 'Darwin':

# took md5sum off for Darwin with wget 2016-01-09 - currie

#		cmd = "which md5"

#		txt1 = os.popen( cmd ).read().strip()
#		returntxt = txt1.split()

#		if len( returntxt ) == 0 :
#			mfail = 'T'
#		else:
#			print("Darwin found md5 @ %s - OK" % ( returntxt ) )

		cmd = "which wget"

		txt1 = os.popen( cmd ).read().strip()
		returntxt = txt1.split()

		if len( returntxt ) == 0 :
		
			mfail = 'T'

		else:
			print("Darwin found wget @ %s - OK" % ( returntxt ) )

# DF local drive - print disk capacity



	if mfail == 'T' :

		print("MacOS-Darwin has no wget installed. Please use UNIX/Linux or choose CURL next!\n")

	response0=raw_input("Use CURL? (Y/N - default N = wget) : ")
	
	if response0 == 'Y' or response0 == 'y' :
		
		mfail = 'F'		
		transferCmd = 'curl'


	f=os.statvfs(cwd)
	blocksize=f[statvfs.F_FRSIZE]
	freeblocks=f[statvfs.F_BAVAIL]
	diskfree=blocksize*freeblocks
	df=BytestoText(diskfree)
	
	print( "disk freespace: "+df+" on " + cwd + "\n" )

	quser, qpw, qdir, qfiles, qinstr = getQueryDat( cwd )


		

	while True :
		
#		print( "\n- Count Current FITS Files in Download Directory -\n" )

		
		fullpath = cwd + '/'


		if os.path.exists( fullpath ):
		
			dirfiles = os.listdir( fullpath )
			dirfiles2 = []
			for filename in dirfiles :
			
				if filename.endswith ('.fits') :
				
					dirfiles2.append( filename )

			fv = len (dirfiles2)

#			os.chdir ( fullpath )

#			cmd = 'ls -al *.fits | wc -l'								
#			fv = os.popen(cmd).read().strip()
#			fv = fv.strip()
			

		else:
			fv = '0'

		os.chdir ( cwd )


	#	if int( fv ) > 0 :

	# calculate remaining files from ( query.dat - filecount )

		remainFiles = int( qfiles ) - int( fv )

		print( "FITS found: " + str(fv) + " of " + str( qfiles ) + " Remain: " + str( remainFiles ) )
#		print( str(qfiles) + " Total FITS Files are listed for this Query." )
#		print( str(remainFiles) + " FITS Files Missing to be downloaded.\n" )
		
		
#		print( "\n- Count Current CRX Files -\n" )

		correxfile = 'fitscorrex.txt'   

		inputcorrex  = cwd + '/zadmin/' + correxfile
		
		correxseq = 0
		
		fv = 0

		if os.path.exists ( inputcorrex ) :

	#open correxlist.txt
			FILE30 = open( inputcorrex ,"r")
			listfiles30 = FILE30.readlines()
			FILE30.close()


			for line in listfiles30:

				correxseq += 1



		fullpath = cwd + '/zcorrex'


		if os.path.exists( fullpath ):
		
			dirfiles = os.listdir( fullpath )
			dirfiles2 = []
			for filename in dirfiles :
			
				if filename.endswith ('.crx') :
				
					dirfiles2.append( filename )

			fv = len (dirfiles2)


		remainFiles = int( correxseq ) - fv
		
		
		print( "CRX files found: " + str(fv) + " of " + str( correxseq ) + " Remain: " + str( remainFiles ) + "\n")

#		print( str(fv) + " CRX Files exist in " + fullpath )
#		print( str(qfiles) + " Total CRX Files are listed for this Query." )
#		print( str(remainFiles) + " CRX Files Missing to be downloaded.\n" )
		
		
		os.chdir ( cwd )


	# 1st Pass - query.dat information

		qanon='anon-OK'



#		print( "colleagues:\n" )

		if correxseq > 0 :

			print( "FITS Corrections Exist! ( " + str( correxseq ) + " ).\nYou can Download FITS Corrected or ASCII Corrections" )
			print( "MD5 checksums are included for both FITS Original and FITS Corrected" )
			print( "Choose 1) Download FITS Original or 2) Download FITS Corrected" + "\n")
			print( "After download of ASCII Corrections, you may choose 6) Update Local FITS." + "\n")
		else:
			print( "No FITS Corrections! ( 0 )\n" )
			print( "MD5 checksums are included for FITS Original" )
	#		print( "Choose: 1) Download Corrected FITS or 2) Download Original FITS + Corrections" )
			print( "Choose 1) Download FITS Original" + "\n")
			

#		print( "OR\nYou may choose to 2) Download the Original FITS files and FITS Corrections as separate ASCII files)" )
#		print( "****\nAfter #2 you may choose to 3) View the Corrections in text" )
#		print( "and 4) Update Local FITS with Corrections\n****\n" )


		if qfiles > 0 :

			print( "1. Download FITS Original (" + str( qfiles ) + ") ./*.fits" )

		if correxseq > 0 :

			print( "2. Download FITS Corrected (" + str( correxseq ) + ") ./*.fits" )
			print( "3. Download ASCII Corrections (" + str( correxseq ) + ") ./zcorrex/*.crx" )

		print( "4. Compare MD5 checksums to confirm Local FITS" )

		if correxseq > 0 :

			print( "5. Display Local ASCII Corrections ./zcorrex/*.crx" )
			print( "6. Update Local FITS with Local Corrections ./zcorrex/*.crx -> ./*.fits" )

		print( "7. Exit" )

		response0=raw_input('Choose : ')

		if response0 == '1':

			print("Download Original FITS + Corrections")
			
			domain = getDomain2()

			quser, qpw, qdir, qfiles, qinstr = getQueryDat( cwd )

			downloadFITS( cwd, domain, quser, qpw, qdir, qfiles, qinstr, transferCmd )


		if response0 == '2':

			print("Download Corrected FITS")

			domain = getDomain2()

			quser, qpw, qdir, qfiles, qinstr = getQueryDat( cwd )

			downloadFrameCrx( cwd, domain, quser, qpw, qdir, transferCmd )

		if response0 == '3':

			print("Download ASCII Corrections")
			
			domain = getDomain2()

			quser, qpw, qdir, qfiles, qinstr = getQueryDat( cwd )

			downloadCrx( cwd, domain, quser, qpw, qdir, qfiles, qinstr, transferCmd )

		if response0 == '4':

			print( "Compare MD5s - Check Local FITS" )
			
			compareMD5( cwd )

		if response0 == '5':

			print( "View Local Corrections" )
			
			displayCorrex( cwd )
			
		if response0 == '6':

			print( "Update Local FITS with Local Corrections ( *.crx -> *.fits )" )
			
			dirfiles = checkCRXDir ( cwd )

			start_time = time.time()

			seq = 0



			for filename in dirfiles :
				seq += 1


		#	filename = 'HSCA90939557'
		#	filename = 'HSCA90939556'
				filename = filename [ 0:12 ]

	# read Corrections Lines from .crx

				CorrectLines = getCorrex( cwd, filename )

				print("CorrectLines: " + str( CorrectLines ) )

			#	Corrections = CorrectLines.split( "\n" )

			#	for line in Corrections:

			#		print "check:" + line


	# validate Corrections


				validateStatus = validateCorrex ( cwd, filename, CorrectLines )

				print ("Validate status: True = Write crx " + str( validateStatus )  +  " " + filename)

				if validateStatus == True  :


	# write Corrections to FITS
					writeCorrex ( cwd, filename, CorrectLines )

	# Rename FITS File
					renameCorrex ( cwd, filename )

			elapsed_time = int(time.time() - start_time)
			print ("Number of corrections : " + str(seq)) 
			print ("Elapsed seconds : " + str(elapsed_time) )
			if elapsed_time > 0 :

				print ("Corrections per second: " + str( seq / elapsed_time )  )


		if response0 == '7':

			break 
	

def validateCorrex ( cwd, filename, CorrectLines ) :

	grab_bytes = 80
	fits_bytes = 0
	lineseq = 0

	filename = cwd + filename + '.fits'
		
	result = False
	
	kwCorrections = getkwCorrections ( CorrectLines )
	
#	kwCorrections = []

#	AllCorrections = CorrectLines.split("\n")

#	for CorrectLine in AllCorrections :
	
#		if len( CorrectLine ) > 0:
		
#			kwCorrect = CorrectLine[0:8]
#			if kwCorrect not in kwCorrections :
#				kwCorrections.append( kwCorrect )

	FILE2 = open ( filename, 'rb' )
#	listfiles = FILE2.readlines()
		
	while True :

		lineseq += 1

		listfiles2 = FILE2.read( grab_bytes )

		if not listfiles2:

			break

		fits_bytes += grab_bytes

		lineLength = len( listfiles2 )

#			print( listfiles2 )

#			print("This " + str( lineseq )  + " line is : "+str( lineLength ) )

		kw = listfiles2[0:8]

		if not kw in kwCorrections:

			tom = 1
			#print ('No update ' + listfiles2 )
					
		else: 

			AllCorrections = CorrectLines.split("\n")

			for FixLine in AllCorrections :

				if len ( FixLine ) > 0 :

					kwCorrect = FixLine[0:8]

					if kwCorrect == kw :

						CorrectLine = FixLine.strip()	
						
			crxFragments = CorrectLine.split("=")

			keyword1 = crxFragments[0]
			values1 = crxFragments[1]

			newvalue = values1.strip()

			fitsFragments = listfiles2.split("=")

			keyword2 = fitsFragments[0]
			values2 = fitsFragments[1]
			oldvalue = values2.strip()
			strpos = oldvalue.find(' / ')

			if strpos > 0 :

				oldvalue = oldvalue [0:strpos]

				oldvalue = oldvalue.strip()	
				
			transtab = maketrans( '"', "'" )
			oldvalue = oldvalue.translate( transtab )	
			newvalue = newvalue.translate( transtab )	

			
			result = checkValues ( oldvalue, newvalue )
					

	FILE2.close()
	
	return ( result ) 
	
def downloadFrameCrx ( cwd, domain, quser, qpw, qdir, transferCmd ) :

	datafile='/zadmin/fitscorrex.txt'
	
	fullpath = cwd + datafile
	
	datafile2='/zadmin/filelist.txt'
	
	fullpath2 = cwd + datafile2

	FILE1 = open( fullpath2, 'r' )
	frame_original = FILE1.readlines()
	FILE1.close()
 
#write to transfer file


	inboundCrx ='/zadmin/inboundCrx.txt'
	crxpath = cwd + inboundCrx
	FILE2 = open( crxpath, 'w' )

	inbound ='/zadmin/inbound.txt'
	originalpath = cwd + inbound
	FILE3 = open( originalpath, 'w' )
	
	
	mnocheck = ' --no-check-certificate'

	correx_count = 0
	
	if os.path.exists( fullpath ):

		print( "\n" )

#read from corrections list			

		frame_crx = open( fullpath, 'r' )
 
		for line in frame_crx:
		
			correx_count += 1
		
			frameID = line[0:12]

#			FILE3.write( domain + qdir + "/" + frameID2 + ".fits\n" )

			if transferCmd == 'curl' :
			
				wgetline = "curl -u "+ quser + ":" + qpw + " -O " + domain + qdir + "/zcorrex/" + frameID + ".crx\n"
		
			else:
				wgetline = domain + qdir + "/zcorrex/" + frameID + ".fits\n"			

#			wgetline = "wget --user=" + quser + " --password=" + qpw + mnocheck + " '" + domain + qdir + "/zadmin/frame_crx.py?frameID=" + frameID + "&execDL=yes' -O " + frameID + ".fits -a ./zadmin/transfer.log\n"
#			print wgetline
			FILE2.write ( wgetline )
			

	FILE2.close()
		
	
	if correx_count > 0:
	
		print("Corrected FITS = ( " + str( correx_count ) + " ) downloading now." )
	
		if transferCmd == 'curl' :

			cmd = 'chmod 755 ' + crxpath
			os.system( cmd )
			cmd = crxpath
			os.system( cmd )
		
		else:
		
			wgetcmd = 'wget --user=' + quser + ' --password=' + qpw + mnocheck + ' -i .' + inboundCrx + ' -nv  -a ./zadmin/transfer.log' 

#			print( "wgetcmd1: " + wgetcmd )

			os.system( wgetcmd )	


	FILE2 = open( crxpath, 'r' )
					
	frameinbound = FILE2.readlines()

	FILE2.close()
	
	original_count = 0
	
	for line in frame_original :
	
		frameID2 = line[0:12] 
		
		found = False
		
		for line2 in frameinbound :
		
			if frameID2 in line2 :
			
				found = True
				
		if found == False :
			
			original_count += 1
		
#			FILE3.write = open( originalpath, 'w' )

			if transferCmd == 'curl' :
			
				wgetline = "curl -u "+ quser + ":" + qpw + " -O " + domain + qdir + "/zcorrex/" + frameID + ".crx\n"
				
			else:
				wgetline = domain + qdir + "/" + frameID2 + ".fits\n"
			
			FILE3.write( wgetline )
	
	FILE3.close()	

#	cmd='chmod 755 .' + inboundCrx
#	os.system( cmd )				

#	cmd='.' + inboundCrx
#	os.system( cmd )				
	
	if original_count > 0:
	
		print( "FITS Original = ( " + str( original_count ) + " ) downloading now." )

		if transferCmd == 'curl' :

			cmd = 'chmod 755 ' + originalpath
			os.system( cmd )
			cmd = originalpath
			os.system( cmd )
		else:
		
			wgetcmd = 'wget --user=' + quser + ' --password=' + qpw + mnocheck + ' -i .' + inbound + ' -nv -a ./zadmin/transfer.log' 
#			print( "wgetcmd2: " + wgetcmd )
			os.system( wgetcmd )	
		
def downloadCrx ( cwd, domain, quser, qpw, qdir, qfiles, qinstr, transferCmd ) :

	netfile = 'inbound.txt'
	
	mnocheck=' --no-check-certificate'
	
	filepath2 = cwd + '/'

	correxfile = 'fitscorrex.txt'   

	inputcorrex  = cwd + '/zadmin/' + correxfile

	if os.path.exists ( inputcorrex ) :

#open correxlist.txt
		FILE30 = open( inputcorrex ,"r")
		listfiles30 = FILE30.readlines()
		FILE30.close()

		correxseq = 0

		for line in listfiles30:

			correxseq += 1

		if correxseq > 0 :

			inputpath2=cwd+'/zadmin/' + netfile
			today = datetime.datetime.today()
			todayString=today.strftime('%y%m%d')
			# open new netfile_YYMMDDD
			netfile2= netfile + ".Crx." + todayString

			inputpath3= cwd + '/zadmin/' + netfile2 

			#						print 'chkfits inputpath2: ' + inputpath2 + ' inputpath3: ' + inputpath3									
			FILE3 = open( inputpath3 ,"w")

			for line in listfiles30 :
			# make values
			#								fid2, fsize2, fmd5, fdate_obs =line.split(' ')
				filedata=line.split(' ')

				fid2 = filedata[0]
				fsize2 = filedata[1]
				fmd5 = filedata[2]
				fdate_obs = filedata[3]

				if transferCmd == 'curl' :
				
					FILE3.write( "curl -u "+ quser + ":" + qpw + " -O " + domain + qdir + "/zcorrex/" + fid2 + ".crx\n" )
				
				else: 
				
					FILE3.write( domain + qdir + "/zcorrex/" + fid2 + ".crx\n" )


			FILE3.close()

			filepath2 = cwd + '/zcorrex/'

			print( 'filepath2: ' + filepath2 )

			os.chdir ( filepath2 )
			
			if transferCmd == 'curl' :
			
							
				netfilePath2 = './zadmin/' + netfile2

				cmd = 'chmod 755 ' + netfilePath2
				os.system( cmd )
				cmd = netfilePath2
				os.system( cmd )

			else:

				cmd='wget --user='+quser+' --password=' + qpw + mnocheck + ' -i ../zadmin/' + netfile2 + ' -nv -a ../zadmin/transfer.log' 

				print("wgetCmd: "+cmd )
				

			os.system( cmd )

			print( "Corrections Download Completed" )				

			os.chdir ( cwd )

		else :

			print ("No more corrections")


def downloadFITS( cwd, domain, quser, qpw, qdir, qfiles, qinstr, transferCmd ) :

# cwd - users current working dir, domain = downloadsource, query.dat: user, pw, download dir, instr

# move S2query.tar to ./zadmin

	datafile='/S2Query.tar'
	fullpath=cwd+datafile
	newpath=cwd+'/zadmin/S2Query.tar'
	if os.path.exists(fullpath):

		print( "Moving ./S2Query.tar to ./zadmin/S2Query.tar" )
		shutil.move( fullpath, newpath )


	
# Choose Network Route and protocol

	mnocheck=' --no-check-certificate'

# download updated wget input file

# default to Not Updateing unpack,py

	response20 = 'N'
	

# dont update unpack.py

#	testing = 'twin'

	testing = 'No'
	
	mfail = 'F'

	netfile = 'inbound.txt'
	
	if testing == 'twin' :
		
		response10=raw_input('Confirm unpack.py MD5?  Usually, select N (y/n): ')
	
	
		if response10 == 'y' or response10 == 'Y' :


			md5fail = 'F' 
			md5txt = ''

			if uname == 'Darwin':

		# took md5sum off for Darwin with wget 2016-01-09 - currie

				cmd = "which md5"

				txt1 = os.popen( cmd ).read().strip()
				returntxt = txt1.split()

				if len( returntxt ) == 0 :

					md5fail = 'T'

				else:
					print("Darwin found md5 @ %s - OK" % ( returntxt[0] ) )

					md5txt = returntxt[0] + ' -r'


			else:

				cmd = "which md5sum"

				txt1 = os.popen( cmd ).read().strip()
				returntxt = txt1.split()

				if len( returntxt ) == 0 :

					md5fail = 'T'

				else:

					print("*NIX found md5sum @ %s - OK" % ( returntxt ) )
					md5txt = returntxt[0] + ' -b'



			if md5fail == 'F' and len( md5txt ) > 0 :

				newdir=cwd + '/zadmin/'

				os.chdir( newdir )

				domain = getDomain ( netfile )

				response20 = 'N'

				if not domain == 'none' :

					md5filename = newdir + 'unpack.md5'

					if os.path.exists( md5filename ):

						os.remove( md5filename )

					netfileInput = domain + qdir + "/zadmin/zadmin/unpack.md5"											

					cmd='wget --user='+quser+' --password=' + qpw + mnocheck + " " +netfileInput+" -a ./transfer.log"

					os.system(cmd)

					if os.path.exists( md5filename ) :

						gmd5=open( md5filename,'r' )

						for line in gmd5 :

							md5data = line.split()
							md5value = md5data[0].strip()

						gmd5.close()

						cmd = md5txt + ' ./unpack.py'

						txt1 = os.popen( cmd ).read().strip()

						returntxt = txt1.split()

						if len( returntxt ) > 0 :

							md5real = returntxt[0].strip()

							if md5real.upper() == md5value.upper() :

								print( "unpack.py == unpack.md5, current unpack.py script may continue.\n" )


							else:

								response20 = 'Y'		

					else :
					
						print( "unpack.py MD5 from STARS is not available. current unpack.py script may continue.\n" )


# dont uupdate unpack.py

#	response20 = 'N' | this is above!

# 181005 THIS IS DISABLED!
	
	if response20 == 'Y':
				
		response21=raw_input('Your unpack.py requires updating. Update Now? (y/n): ')
		
		if response21 == 'Y' or response21 == 'y': 

	# change dir to /zadmin for the netfile download


			mfail = 'T' 
								
			newdir=cwd + '/zadmin/'

			os.chdir( newdir )

			domain = getDomain ( netfile )

			if not domain == 'none' :

				unpackfilename = newdir + 'unpack.py'
			
				if os.path.exists( unpackfilename ):

					os.remove( unpackfilename )
					
#			cmd='wget -N --user='+quser+' --password='+qpw+mnocheck+ " " +netfileInput+" -a ./transfer.log"

				print("Please restart unpack.py after this download-update!\n************************")

				netfileInput = domain + qdir + "/zadmin/zadmin/unpack.txt"
															
				cmd='wget --user='+quser+' --password=' + qpw + mnocheck + " " +netfileInput+" -a ./transfer.log"
				os.system(cmd)

				os.rename('./unpack.txt', './unpack.py')

				cmd='chmod 755 ./unpack.py'

				os.system( cmd )				
				
				os.chdir(cwd)
				
				os.execv(__file__,sys.argv)
				

# change dir back
											
	os.chdir( cwd )


# if we did not update unpack.py

	if mfail == 'F':


# 2nd pass - query information after update

		qanon='anon-OK'


		print( "\n- Count Current Files in Download Directory -\n" )

		
		fullpath = cwd + '/'


		if os.path.exists( fullpath ):

			os.chdir ( fullpath )

			cmd = 'ls -al *.fits | wc -l'								
			fv = os.popen(cmd).read().strip()
			fv = fv.strip()
		else:
			fv = '0'

		os.chdir ( cwd )


	#	if int( fv ) > 0 :

	# calculate remaining files from ( query.dat - filecount )

		remainFiles = int( qfiles ) - int( fv )

		print( "( " + str( fv ) + " ) FITS Files exist in " + cwd )
		print( "( " + str( qfiles ) + " ) Total FITS Files listed for this Query." )
		print( "( " + str( remainFiles ) + " ) FITS Files Missing to be downloaded.\n" )

	# is anonymous download allowed? anon-NG - enter personal login/pw

		if qanon == 'anon-NG' :

			print( "No Anonymous Download allowed: " + qanon )	
			response50=raw_input('Enter Personal STARS Login: ')
			quser=response50.strip()
			response51=raw_input('Enter Personal STARS Password: ')
			qpw=response51.strip()

	# if existing files > 0, then confirm MD5s?
		

#		if int( fv ) > 0 :

		noMissingMD5 = 'T'
		
		if noMissingMD5 == 'F' :
		
			existingFiles=str( fv )	
			confirmText= "Confirm MD5s on " + existingFiles + " Existing Files?"
			response40=raw_input('Confirm MD5s on existing Files? (y/n)')
			if response40 =='Y' or response40 == 'y' :
			
				missCount, failCount, goodCount = checkMD5s( cwd, qdir, netfile, quser, qpw, mnocheck )

	# if remaining files in download directory > 0

			if remainFiles > 0 :

				response20=raw_input('List Missing files? (y/n)')

				if response20=='y' or response20=='Y':

					listpath30 = cwd+'/zadmin/filelist.txt'
					if os.path.exists( listpath30 ):

			# open filelist.txt
						FILE30 = open( listpath30 ,"r")
						listfiles30 = FILE30.readlines()
						FILE30.close()

						missingSeq = 0			

						for line in listfiles30:
		# make values
							fid2, fsize2, fmd5, fdate_obs =line.split(' ')
							fid=fid2.strip()
							fsize=int( fsize2.strip() )
							fmd5 = fmd5.strip()
							fmd5 = fmd5.upper()

							if qinstr == 'HSCSSP':
								filepath2 = cwd + '/' + qdate + '/' + fid + '.fits'
							else:
								filepath2 = cwd + '/' + fid + '.fits'

	#						filepath2 = cwd + '/' + fid + '.fits'
		#
		#					print 'chkfits filepath2' + filepath2 									
		#					mGet = False
		# check Not os.path.exists
							if not os.path.exists( filepath2 ):
								missingSeq = missingSeq + 1							
								print( str(missingSeq) +" - Missing: " + fid + ".fits" )

					if missingSeq > 0:

						print( "\n* To Download the Above-Listed Missing Files Only:" )
						print( "* choose 'Continue Wget Preparation?' = Y" )
		#				print "* choose 'Network Route' number"
						print( "* choose 'Select FrameID Range to Download?' = N" )
						print( "* choose 'Download Missing/Error FITS Files?' = Y" )
						print( "* choose 'Confirm MD5s values for Existing Files?' = N" )
						print( "* choose 'Fix MD5 Errors - Delete Errors & re-Download?' = N\n" )
		#		else:
		#			mfail = 'T'


	# confirm disk space wget
	#	print '\n'
	#	print '-- Transfer DiskSpace Check --'
	##	print '\n'

	#	qs=BytestoText(qcompsize)
	#	print 'diskfree: ' + str( diskfree )
	#	print 'qcompsize: ' + str( qcompsize )
	#	if int( diskfree ) > int( qcompsize ):
	#		print 'You have free space to transfer: '+qs+' / '+df
	#	else:
	#		print 'You have NO free space to transfer: '+qs+' / '+df
	#		print 'You have NO free space to transfer: '+qs+' / '+df
	#		mfail='T'


	#	print( "above mfail: "+mfail )	
	# wget


# is there an existing downlaod running

	wgetFound, wgetFoundText = wGetRunning()
	curlFound, curlFoundText = curlRunning()
	
	if wgetFound == True or curlFound == True:
	
		mfail = 'T'

		print("\n************************")
		print("Critical Error! You have a wget/curl process currently-running!")
		print("This is a common problem. 'ps -ef | grep wget/curl' reports:")
		print( wgetFoundText, curlFoundText )
		print("The unpack.py script runs without a progress bar and downloads with no notice.")
		print("Look for files in the download directory, and if new files are not appearing\nregularly, then kill the running wget/curl process and restart unpack.py.")
		print("Otherwise - if files are appearing in the download dir regularly, then let the currently-running wget/curl finish.")
		print("************************")

	start_time = time.time()

	if mfail == 'F' and not netfile == 'fitscorrex.txt' :
	
#		print( "past mFail = T" )
#	
		response1=raw_input('Prepare to Download? (y/n)')

		zipfzCount = 0
		fzCount = 0
		fitsCount = 0
		
		listpath30 = cwd+'/zadmin/filelist.txt'

		if os.path.exists( listpath30 ):

			# open filelist.txt
			FILE30 = open( listpath30 ,"r")
			listfiles30 = FILE30.readlines()
			FILE30.close()


			# for line in filelist.txt

			for line in listfiles30:
			# make values
			      fid2, fsize2, fmd5, fdate_obs, fzip = line.split(' ')
			      fid=fid2.strip()
			      fsize=int( fsize2.strip() )
			      fmd5 = fmd5.strip().upper()
			      fzip = fzip.strip()

			      if fzip == 'zipfz' : 
		      		zipfzCount += 1
				
			      if fzip == 'fz' :
		      		fzCount += 1
				
			      if fzip == 'fits' :
		      		fitsCount += 1
		

		if qinstr == 'HSC' and ( zipfzCount > 0 ):
		
			responseZip=raw_input('HSC Compressed (FPACK) found - Download Compressed? (y/n)')
		
		else: 
			responseZip = 'N'

#		responseZip = 'N'
		
			
		
		if response1=='y' or response1=='Y':
		

			
# write inbound.txt with domain + qdir + filelist files +  .fit
		
#			domain = getDomain ( netfile )

			filelist = '/zadmin/filelist.txt'
			fullpath = cwd + filelist
			outfilelist = '/zadmin/inbound.txt'
			fullpathout = cwd + outfilelist						


			f = open( fullpathout,'w')

#			print("fullpath: "+fullpath)
#			print("fullpathout: "+fullpathout)
		
			loglist=[ 'TSC', 'END', 'TAS', 'TAD', 'STA', 'ENV', 'QRE', 'QCA' ]			
			
			if os.path.exists( fullpath ) :

#				print("fullpath Exists: "+fullpath)
			
				g=open( fullpath, 'r')				
				listfiles=g.readlines()
				g.close()
#				print("listfiles: "+str(listfiles))				
				for line in listfiles:

					filedata = line.split()
					qframeid = filedata[0]
					qfprefix = qframeid[0:3]
					if len ( filedata ) > 4:
					
						zipStatus = filedata[4]
					else:
						zipStatus = 'fits'
					
#					print('writing: domain-' + domain + ' qdir-'+ qdir + ' qframeid-'+ qframeid +	".fits")

					if qfprefix in loglist or zipStatus == 'log' :	
										
						if transferCmd == 'curl' :

							f.write( "curl -u " + quser + ":" + qpw + " -O " + domain + qdir + "/" + qframeid + "\n" )
						else:
							f.write( domain + qdir + "/" + qframeid + "\n" )
						
					else:
					
#						if responseZip == 'Y' or responseZip == 'y' :						
#							f.write( domain + qdir + "/zip/" + qframeid + ".fits.fz\n" )
						#						else: 					
#							f.write( domain + qdir + "/" + qframeid + ".fits\n" )

						if transferCmd == 'curl' :

							f.write( "curl -u " + quser + ":" + qpw + " -O " + domain + qdir + "/" + qframeid + ".fits\n" )
						else: 
						
							if zipStatus == 'fits' :

								f.write( domain + qdir + "/" + qframeid + ".fits\n" )

							if zipStatus == 'zipfz' :

								if responseZip == 'Y' or responseZip == 'y' :

									f.write( domain + qdir + "/zip/" + qframeid + ".fits.fz\n" )

								else :

									f.write( domain + qdir + "/" + qframeid + ".fits\n" )

							if zipStatus == 'fz' :

								f.write( domain + qdir + "/" + qframeid + ".fits.fz\n" )
							
						
			else:
			
				print("fullpath Not Exists: "+fullpath)

			f.close()
			
		
#			print '\n'				
			

# Start / End Frame_ID

			response6=raw_input('Start Download from one FrameID? (n = ALL FrameIDs) (y/n): ')

			if response6 == 'y' or response6 == 'Y' :

				response7=raw_input('Enter Start FrameID: ')

				if len( response7 ) == 12 :

					response9=raw_input('Enter End FrameID (blank for All): ')

					endFrame = ''

					if len( response9 ) == 12 :
						endFrame = response9

					restartFrameID = response7
					inputpath2=cwd+'/zadmin/' + netfile	
					today = datetime.datetime.today()
					todayString=today.strftime('%y%m%d')

					netfile2= netfile + "." + todayString	

					inputpath3=cwd+'/zadmin/' + netfile2
					print( "inputpath2: " + inputpath2 + " inputpath3: " + inputpath3 )									
					if os.path.exists( inputpath2 ):

						FILE1 = open( inputpath2 ,"r")
						listfiles2 = FILE1.readlines()
						FILE1.close()

						FILE2 = open( inputpath3 ,"w")

						seq = 0
						startHere = 'No'
						endHere = 'No'

						for line in listfiles2:

							seq += 1

#							print str(seq) + ' '+ line

							if response7 in line:

								print("* Start FrameID Found: "+response7 + " startHere=Yes" )

								startHere = 'Yes'

							if startHere == 'Yes' and endHere == 'No':
							
								if transferCmd == 'curl' :
								
									line = 'curl -u '+ quser + ":" + qpw + " -O " + line 
								
								FILE2.write( line )

							if endFrame != '' and endFrame in line:

								print("* End FrameID Found: "+ endFrame + " endHere=Yes" )

								endHere = 'Yes'




						FILE2.close()

						if startHere == 'Yes':

							print("Selected Network Routing is: " + domain + ": " + netfile2 + " "+mnocheck+"\n")
							print("Transfer log is appended to ./zadmin/transfer.log\n\n")
							print("FITS transferring now, please wait ...\n")

#							if qinstr == 'HSCSSP':
#								cmd='wget --user='+quser+' --password=' + qpw + mnocheck+' -i ./zadmin/'+netfile2+' -a ./zadmin/transfer.log' + ' -P '+qdate
#							else: 
							
							if transferCmd == 'curl' :
							
								netfilePath2 = './zadmin/' + netfile2

								cmd = 'chmod 755 ' + netfilePath2
								os.system( cmd )
								cmd = netfilePath2
								os.system( cmd )
							
							else: 
								cmd='wget --user='+quser+' --password=' + qpw + mnocheck+' -i ./zadmin/'+netfile2+' -nv -a ./zadmin/transfer.log'								
								os.system(cmd)
						else:

							print("Start FrameID NotFound: " + response7 + "\n" )



				else:
					resp7len = len( response7 )
					print("Start FrameID is not 12 characters: (" + str( resp7len ) + ") - " + response7 )
					
						
# Not Start/End, so eitiher a) Missing/MD5 Fails or b) HSC-SSP Provide Date or c) Download All

			else:


# a) Missing/MD5 Fails		
				response10=raw_input('Download Missing/MD5Fails files ONLY? (n = Download ALL) (y/n): ')
			
				if response10 == 'y' or response10 == 'Y' :

					response21=raw_input('Confirm MD5s of Existing Files? (n = only check Missing files)(y/n): ')
					
					response11=raw_input('Fix MD5 Errors - Delete Errors & re-Download? (n = NoDelete/NoDownload) (y/n): ')
					
						
					listpath30 = cwd+'/zadmin/filelist.txt'

					if os.path.exists( listpath30 ):

# open filelist.txt
						FILE30 = open( listpath30 ,"r")
						listfiles30 = FILE30.readlines()
						FILE30.close()

# open existing netfile

						inputpath2=cwd+'/zadmin/' + netfile	
						today = datetime.datetime.today()
						todayString=today.strftime('%y%m%d')
# open new netfile_YYMMDDD
						netfile2= netfile + "." + todayString	

						inputpath3=cwd+'/zadmin/' + netfile2

#						print 'chkfits inputpath2: ' + inputpath2 + ' inputpath3: ' + inputpath3									
						FILE3 = open( inputpath3 ,"w")


						if os.path.exists( inputpath2 ):

# read netfile into array
							FILE1 = open( inputpath2 ,"r")
							listfiles2 = FILE1.readlines()
							FILE1.close()
							
							failCount = 0
# for line in filelist.txt
							for line in listfiles30:
# make values
#								fid2, fsize2, fmd5, fdate_obs =line.split(' ')
								filedata=line.split(' ')
								
								fid2 = filedata[0]
								fsize2 = filedata[1]
								fmd5 = filedata[2]
								fdate_obs = filedata[3]
								
								if len ( filedata ) > 4 :
								
									fzipStatus = filedata[4]
									
								else:
								
									fzipStatus = 'fits'
									
								
								fid=fid2.strip()
								
								fsize=int( fsize2.strip() )
								fmd5 = fmd5.strip().upper()
								

								if qinstr == 'HSCSSP':
								
									filepath2 = cwd + '/' + qdate + '/' + fid + '.fits'
								else:
									filepath2 = cwd + '/' + fid + '.fits'
								
#								print 'chkfits filepath2' + filepath2 									
								mGet = False
# check Not os.path.exists
								if not os.path.exists( filepath2 ):

									mGet = True
									failCount = failCount + 1
									print("Missing: "+filepath2 + " failCount: " + str( failCount ) )
#
# else check filebytes
								else:
									if response21 == 'Y' or response21 == 'y' :
	# filebytes
	#									cmd = 'cksum '+filepath2
	#									fv = os.popen(cmd).read().strip()
	#									realCksum = fv.split(' ')
	#									print 'realCksum: ' + realCksum[0] + ' realCBytes: '+realCksum[1] + ' realFile: '+realCksum[2]
	#									realFSize = int( realCksum[1] )
	#									if not fsize == realFSize:

	# md5 checking									
										cmd = 'md5sum '+filepath2
										fv = os.popen(cmd).read().strip()
										realMD5 = fv.split(' ')
	#									print 'realMD5: ' + realMD5[0] + ' realMD5File: '+realMD5[1]
										actualMD5 = realMD5[0]
										actualMD5=actualMD5.strip().upper()

										if not fmd5 == actualMD5 :

	# if file exists but wrong size - unlink and wget again				
											if response11 == 'Y' or response11 == 'y' :				
												cmd = 'unlink '+ filepath2
												os.system(cmd)

											failCount = failCount + 1
											mGet = True

											print( "* MD5s NoMatch *: " + filepath2  + " failCount: " + str( failCount ) )
											print( fmd5 + " != " + actualMD5 )

										else:
											print("MD5s Match: "+ fid + " == " + actualMD5 )
										

								if mGet == True :
									
									for line2 in listfiles2 :
									
										if fid in line2 :
										
											if transferCmd == 'curl' :
												
												line2 = 'curl -u '+ quser + ":" + qpw + " -O " + line2 
										
											FILE3.write( line2 )
#											print 'writing line2: '+line2
											
																	
																 
						FILE1.close()
						FILE3.close()
						FILE30.close()
						
						print("Total Missing/MD5Fails Count: " + str( failCount ) )
						
						if ( response11 == 'Y' or response11 == 'y' ) and failCount > 0 :
																								
							print( "Selected Network Routing is: "+domain+": "+netfile2+" "+mnocheck+"\n" )
							print( "Transfer log is stored as ./zadmin/transfer.log\n\n" )
							print( "transferring now, please wait ...\n" )

#							if qinstr == 'HSCSSP':
#								cmd='wget --user='+quser+' --password=' + qpw + mnocheck+' -i ./zadmin/'+netfile2+' -a ./zadmin/transfer.log' + ' -P '+qdate
#							else:

							if transferCmd == 'curl' :
							
								netfilePath2 = './zadmin/' + netfile2

								cmd = 'chmod 755 ' + netfilePath2
								os.system( cmd )
								cmd = netfilePath2
								os.system( cmd )
							
							else:
							
								cmd='wget --user='+quser+' --password=' + qpw + mnocheck+' -i ./zadmin/'+netfile2+' -nv -a ./zadmin/transfer.log'

								os.system(cmd)
								

				else:
									

					print( "Selected Network Routing is: "+domain+": "+netfile+" "+mnocheck+"\n" )
					print( "Transfer log is stored as ./zadmin/transfer.log\n\n" )
					print( "transferring now, please wait ...\n" )

					if transferCmd == 'curl' :									

						netfilePath = './zadmin/' + netfile
						
						cmd = 'chmod 755 ' + netfilePath
						os.system( cmd )
						cmd = netfilePath
						os.system( cmd )
											
					else:
						cmd='wget --user=' + quser + ' --password=' + qpw + mnocheck + ' -i ./zadmin/'+netfile+' -nv -a ./zadmin/transfer.log'					
						os.system( cmd )

		else:

			print( "You chose to NOT continue Wget Preparation\n" )



	elapsed_time = int( time.time() - start_time )
	
	print ("Elapsed download in seconds: " + str( elapsed_time ) )

	
	filepath2 = cwd + '/'

	correxfile = 'fitscorrex.txt'	
	
	inputcorrex  = cwd + '/zadmin/' + correxfile
				
	if os.path.exists ( inputcorrex ) :

# open correxlist.txt
		FILE30 = open( inputcorrex ,"r")
		listfiles30 = FILE30.readlines()
		FILE30.close()
		
		correxseq = 0
		
		for line in listfiles30:
		
			correxseq += 1
			
		if correxseq > 0 :
		
			print ("Corrections Exists: " + str ( correxseq ) ) 
			
			responseCorrex = raw_input( "Download Corrections? y/n " )
			
			if responseCorrex == 'Y' or responseCorrex == "y" :
			

				inputpath2=cwd+'/zadmin/' + netfile
				today = datetime.datetime.today()
				todayString=today.strftime('%y%m%d')
		# open new netfile_YYMMDDD
				netfile2= netfile + ".Crx." + todayString

				inputpath3= cwd + '/zadmin/' + netfile2 

#						print 'chkfits inputpath2: ' + inputpath2 + ' inputpath3: ' + inputpath3									
				FILE3 = open( inputpath3 ,"w")

				for line in listfiles30:
		# make values
		#								fid2, fsize2, fmd5, fdate_obs =line.split(' ')
					filedata=line.split(' ')

					fid2 = filedata[0]
					fsize2 = filedata[1]
					fmd5 = filedata[2]
					fdate_obs = filedata[3]
					if transferCmd == 'curl' :
					
						FILE3.write( "curl -u " + quser + ":" + qpw + " -O " + domain + qdir + "/zcorrex/" + fid2 + ".crx\n" )
					
					else:
					
						FILE3.write( domain + qdir + "/zcorrex/" + fid2 + ".crx\n" )
					

				FILE3.close()
				
				filepath2 = cwd + '/zcorrex/'
				
				print( 'filepath2: ' + filepath2 )
				
				os.chdir ( filepath2 )
				
				if transferCmd == 'curl' :

					netfilePath2 = './zadmin/' + netfile2

					cmd = 'chmod 755 ' + netfilePath2
					os.system( cmd )

					cmd = netfilePath2
					os.system( cmd )
				
				else:
				
					cmd='wget --user='+quser+' --password=' + qpw + mnocheck + ' -i ../zadmin/'+netfile2+' -nv -a ../zadmin/transfer.log' 
				
					print("wgetCmd: "+cmd )
					
					os.system( cmd )
				
				print( "Corrections Download Completed" )	
				
				filelist=os.listdir( filepath2 )
				
				docorrex='F'
				
				seq=0
				
				correxfiles = len( filelist )
				
				print("Corrections Downloaded to " + filepath2 + "{FrameID}.crx : " + str( correxfiles ) )  
				
				correxseq = 0
				
				for item in filelist:
				
					correxseq += 1
				
					print( str(correxseq) + " " + item )
										
								

			else:
			
				print( "Dont Download Corrections!" )									


		else:

			print ("No Corrections in File. Corrections = " + str( correxseq ) )

	else:

		print ("Corrections File Not Exists: " + inputcorrex )

		
	print("\n")
	print("- STARS Unpack COMPLETED -")
	print("\n")

	os.chdir ( cwd )
	

if __name__ == '__main__':

	main()
